@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-page-action.c,v 1.1 2005/01/04 11:39:08 marco Exp $
+ *  $Id: ev-page-action.c,v 1.2 2005/01/13 16:23:29 andersca Exp $
  */
 
 #include "config.h"
@@ -84,7 +84,7 @@ update_spin (GtkAction *action, gpointer dummy, GtkWidget *proxy)
 
 	value = gtk_spin_button_get_value (GTK_SPIN_BUTTON (spin));
 
-	if (value != page->priv->current_page)
+	if (value != page->priv->current_page )
 	{
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin),
 					   page->priv->current_page);
@@ -101,6 +101,14 @@ value_changed_cb (GtkWidget *spin, GtkAction *action)
 	g_signal_emit (action, signals[GOTO_PAGE_SIGNAL], 0, value);
 }
 
+static void
+total_pages_changed_cb (EvPageAction *action, GParamSpec *pspec,
+			GtkSpinButton *spin)
+{
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (spin), 1, 
+				   action->priv->total_pages);
+}
+
 static GtkWidget *
 create_tool_item (GtkAction *action)
 {
@@ -118,6 +126,9 @@ create_tool_item (GtkAction *action)
 	g_object_set_data (G_OBJECT (item), "spin", spin);
 	gtk_widget_show (spin);
 
+	g_signal_connect (action, "notify::total-pages",
+			  G_CALLBACK (total_pages_changed_cb),
+			  spin);
 	g_signal_connect (spin, "value_changed",
 			  G_CALLBACK (value_changed_cb),
 			  action);
